@@ -1,10 +1,10 @@
-"""Provides EntityDatabase, a class that keeps track of spec-defined entities and associated macros."""
-
+# Copyright 2018-2025 The Khronos Group Inc.
 # Copyright (c) 2018-2019 Collabora, Ltd.
 #
 # SPDX-License-Identifier: Apache-2.0
 #
 # Author(s):    Rylie Pavlik <rylie.pavlik@collabora.com>
+"""Provides EntityDatabase, a class that keeps track of spec-defined entities and associated macros."""
 
 from abc import ABC, abstractmethod
 
@@ -182,7 +182,7 @@ class EntityDatabase(ABC):
             self.addEntity(name, 'code', elem=info.elem, generates=False)
 
         else:
-            raise RuntimeError('unrecognized category {}'.format(cat))
+            raise RuntimeError(f'unrecognized category {cat}')
 
     def handleCommand(self, name, info):
         """Add entities, if appropriate, for an item in registry.cmddict.
@@ -274,7 +274,7 @@ class EntityDatabase(ABC):
             tag = 'member'
         else:
             tag = 'param'
-        return data.elem.findall('.//{}'.format(tag))
+        return data.elem.findall(f'.//{tag}')
 
     def getMemberNames(self, commandOrStruct):
         """Given a command or struct name, retrieve the names of each member/param.
@@ -505,7 +505,10 @@ class EntityDatabase(ABC):
 
         # Look up category based on the macro, if category isn't specified.
         if category is None:
-            category = self._categoriesByMacro.get(macro)[0]
+            cats = self._categoriesByMacro.get(macro)
+            if not cats:
+                raise RuntimeError(f"Could not determine category for {macro}")
+            category = cats[0]
 
         if generates is None:
             potential_dir = directory or category
@@ -518,7 +521,7 @@ class EntityDatabase(ABC):
 
         # Don't generate a filename if this entity doesn't generate includes.
         if filename is None and generates:
-            filename = '{}/{}.txt'.format(directory, entityName)
+            filename = f'{directory}/{entityName}.adoc'
 
         data = EntityData(
             entity=entityName,
@@ -569,7 +572,7 @@ class EntityDatabase(ABC):
         # Retrieve from subclass, if overridden, then store locally.
         self._supportExclusionSet = set(self.getExclusionSet())
 
-        # Entities that get a generated/api/category/entity.txt file.
+        # Entities that get a generated/api/category/entity.adoc file.
         self._generating_entities = {}
 
         # Name prefix members
@@ -578,7 +581,7 @@ class EntityDatabase(ABC):
         ) + self.name_prefix[1:]
         # Regex string for the name prefix that is case-insensitive.
         self.case_insensitive_name_prefix_pattern = ''.join(
-            ('[{}{}]'.format(c.upper(), c) for c in self.name_prefix))
+            (f'[{c.upper()}{c}]' for c in self.name_prefix))
 
         self.platform_requires = self.getPlatformRequires()
 
