@@ -1,10 +1,10 @@
-"""Provides MacroCheckerFile, a subclassable type that validates a single file in the spec."""
-
+# Copyright 2018-2025 The Khronos Group Inc.
 # Copyright (c) 2018-2019 Collabora, Ltd.
 #
 # SPDX-License-Identifier: Apache-2.0
 #
 # Author(s):    Rylie Pavlik <rylie.pavlik@collabora.com>
+"""Provides MacroCheckerFile, a subclassable type that validates a single file in the spec."""
 
 import logging
 import re
@@ -13,8 +13,6 @@ from enum import Enum
 from inspect import currentframe
 from typing import Set
 
-from spec_tools.macro_checker import MacroChecker
-
 from .shared import (AUTO_FIX_STRING, CATEGORIES_WITH_VALIDITY,
                      EXTENSION_CATEGORY, NON_EXISTENT_MACROS, EntityData,
                      Message, MessageContext, MessageId, MessageType,
@@ -35,14 +33,28 @@ INTERNAL_PLACEHOLDER = re.compile(
     r'(?P<delim>__+)([a-zA-Z]+)(?P=delim)'
 )
 
+# Matches any include line.
+# Used to check for a leading path attribute.
+INCLUDE_PATH_ATTRIBUTE = re.compile(
+    r'include::(\{(?P<path_attribute>[a-z]+)\})?.*\[\]')
+
+allowed_path_attributes = {
+    '{appendices}',
+    '{chapters}',
+    '{config}',
+    '{generated}',
+    '{promoted}',
+    '{style}',
+}
+
 # Matches a generated (api or validity) include line.
 INCLUDE = re.compile(
-    r'include::(?P<directory_traverse>((../){1,4}|\{generated\}/)(generated/)?)(?P<generated_type>(api|validity))/(?P<category>\w+)/(?P<entity_name>[^./]+).txt[\[][\]]')  # noqa
+    r'include::(?P<directory_traverse>((../){1,4}|\{generated\}/)(generated/)?)(?P<generated_type>(api|validity))/(?P<category>\w+)/(?P<entity_name>[^./]+).adoc[\[][\]]')  # noqa
 
 # Matches an [[AnchorLikeThis]]
 ANCHOR = re.compile(r'\[\[(?P<entity_name>[^\]]+)\]\]')
 
-# Looks for flink:foo:: or slink::foo:: at the end of string:
+# Looks for flink:foo:: or slink:foo:: at the end of string:
 # used to detect explicit pname context.
 PRECEDING_MEMBER_REFERENCE = re.compile(
     r'\b(?P<macro>[fs](text|link)):(?P<entity_name>[\w*]+)::$')
@@ -50,7 +62,7 @@ PRECEDING_MEMBER_REFERENCE = re.compile(
 # Matches something like slink:foo::pname:bar as well as
 # the under-marked-up slink:foo::bar.
 MEMBER_REFERENCE = re.compile(
-    r'\b(?P<first_part>(?P<scope_macro>[fs](text|link)):(?P<scope>[\w*]+))(?P<double_colons>::)(?P<second_part>(?P<member_macro>pname:?)(?P<entity_name>[\w]+))\b' # noqa
+    r'\b(?P<first_part>(?P<scope_macro>[fs](text|link)):(?P<scope>[\w*]+))(?P<double_colons>::)(?P<second_part>(?P<member_macro>pname:?)(?P<entity_name>[\w]+))\b'  # noqa
 )
 
 # Matches if a string ends while a link is still "open".
@@ -58,7 +70,7 @@ MEMBER_REFERENCE = re.compile(
 # or containing our interested area when matched against the text preceding).
 # Used to skip checking in some places.
 OPEN_LINK = re.compile(
-    r'.*(?<!`)<<[^>]*$'
+    r'.*(?<!`)(<<[^>]*|xref:[^\[]*)$'
 )
 
 # Matches if a string begins and is followed by a link "close" without a matching open.
@@ -73,8 +85,9 @@ CLOSE_LINK = re.compile(
 # - `ifdef:`
 # - `endif:`
 # - `todo` (followed by something matching \b, like : or (. capitalization ignored)
+# - `[#` (anchor for an image)
 SKIP_LINE = re.compile(
-    r'^(ifdef:)|(endif:)|([tT][oO][dD][oO]\b).*'
+    r'^(ifdef:)|(endif:)|([tT][oO][dD][oO]\b)|(\[#).*'
 )
 
 # Matches the whole inside of a refpage tag.
@@ -84,6 +97,7 @@ BRACKETS = re.compile(r'\[(?P<tags>.*)\]')
 REF_PAGE_ATTRIB = re.compile(
     r"(?P<key>[a-z]+)='(?P<value>[^'\\]*(?:\\.[^'\\]*)*)'")
 
+ARRAY_DIMENSION = re.compile(r"\[(?P<dim>[^\]]+)\]$")
 
 class Attrib(Enum):
     """Attributes of a ref page."""
@@ -153,8 +167,8 @@ def _pluralize(word, num):
     if num == 1:
         return word
     if word.endswith('y'):
-        return word[:-1] + 'ies'
-    return word + 's'
+        return f"{word[:-1]}ies"
+    return f"{word}s"
 
 
 def _s_suffix(num):
@@ -184,13 +198,13 @@ def shouldEntityBeText(entity, subscript):
     return False
 
 
-class MacroCheckerFile(object):
+class MacroCheckerFile:
     """Object performing processing of a single AsciiDoctor file from a specification.
 
     For testing purposes, may also process a string as if it were a file.
     """
 
-    def __init__(self, checker: MacroChecker, filename: str, enabled_messages: Set[MessageId], stream_maker):
+    def __init__(self, checker, filename: str, enabled_messages: Set[MessageId], stream_maker):
         """Construct a MacroCheckerFile object.
 
         Typically called by MacroChecker.processFile or MacroChecker.processString().
@@ -268,7 +282,7 @@ class MacroCheckerFile(object):
 
         if self.block_stack:
             locations = (x.context for x in self.block_stack)
-            formatted_locations = ['{} opened at {}'.format(x.delimiter, self.getBriefLocation(x.context))
+            formatted_locations = [f'{x.delimiter} opened at {self.getBriefLocation(x.context)}'
                                    for x in self.block_stack]
             self.logger.warning("Unclosed blocks: %s",
                                 ', '.join(formatted_locations))
@@ -290,8 +304,8 @@ class MacroCheckerFile(object):
 
             if entity not in self.validity_includes:
                 self.warning(MessageId.MISSING_VALIDITY_INCLUDE,
-                             ['Saw /api/ include for {}, but no matching /validity/ include'.format(entity),
-                              'Expected a line with ' + regenerateIncludeFromMatch(includeContext.match, 'validity')],
+                             [f'Saw /api/ include for {entity}, but no matching /validity/ include',
+                              f"Expected a line with {regenerateIncludeFromMatch(includeContext.match, 'validity')}"],
                              context=includeContext)
 
         # Check that we never include a /validity/ file
@@ -299,24 +313,23 @@ class MacroCheckerFile(object):
         for entity, includeContext in self.validity_includes.items():
             if entity not in self.fs_api_includes:
                 self.error(MessageId.MISSING_API_INCLUDE,
-                           ['Saw /validity/ include for {}, but no matching /api/ include'.format(entity),
-                            'Expected a line with ' + regenerateIncludeFromMatch(includeContext.match, 'api')],
+                           [f'Saw /validity/ include for {entity}, but no matching /api/ include',
+                            f"Expected a line with {regenerateIncludeFromMatch(includeContext.match, 'api')}"],
                            context=includeContext)
 
         if not self.numDiagnostics():
             # no problems, exit quietly
             return
 
-        print('\nFor file {}:'.format(self.filename))
+        print(f'\nFor file {self.filename}:')
 
         self.printMessageCounts()
         numFixes = len(self.fixes)
         if numFixes > 0:
-            fixes = ', '.join(('{} -> {}'.format(search, replace)
+            fixes = ', '.join((f'{search} -> {replace}'
                                for search, replace in self.fixes))
 
-            print('{} unique auto-fix {} recorded: {}'.format(numFixes,
-                                                              _pluralize('pattern', numFixes), fixes))
+            print(f"{numFixes} unique auto-fix {_pluralize('pattern', numFixes)} recorded: {fixes}")
 
     def processLine(self, lineNum, line):
         """Check the contents of a single line from a file.
@@ -381,7 +394,21 @@ class MacroCheckerFile(object):
             return
 
         ###
-        # Detect include:::....[] lines
+        # Detect any include:: lines
+        match = INCLUDE_PATH_ATTRIBUTE.match(line)
+        if match:
+            path_attribute = match.group(1)
+            if path_attribute is None:
+                self.error(MessageId.MISSING_INCLUDE_PATH_ATTRIBUTE,
+                           '(no path attribute is present)')
+                return
+            if path_attribute not in allowed_path_attributes:
+                self.error(MessageId.MISSING_INCLUDE_PATH_ATTRIBUTE,
+                           f'(path attribute {{path_attribute}} is not one of {sorted(allowed_path_attributes)})')
+                return
+
+        ###
+        # Detect include:::....[] lines for generated API fragments
         match = INCLUDE.match(line)
         if match:
             self.match = match
@@ -390,7 +417,7 @@ class MacroCheckerFile(object):
             data = self.checker.findEntity(entity)
             if not data:
                 self.error(MessageId.UNKNOWN_INCLUDE,
-                           'Saw include for {}, but that entity is unknown.'.format(entity))
+                           f'Saw include for {entity}, but that entity is unknown.')
                 self.pname_data = None
                 return
 
@@ -410,8 +437,8 @@ class MacroCheckerFile(object):
                     name_and_line = toNameAndLine(
                         self.validity_includes[entity], root_path=self.checker.root_path)
                     self.error(MessageId.API_VALIDITY_ORDER,
-                               ['/api/ include found for {} after a corresponding /validity/ include'.format(entity),
-                                'Validity include located at {}'.format(name_and_line)])
+                               [f'/api/ include found for {entity} after a corresponding /validity/ include',
+                                f'Validity include located at {name_and_line}'])
 
             elif match.group('generated_type') == 'validity':
                 self.recordInclude(self.checker.validityIncludes)
@@ -419,7 +446,7 @@ class MacroCheckerFile(object):
 
                 if entity not in self.pname_mentions:
                     self.error(MessageId.API_VALIDITY_ORDER,
-                               '/validity/ include found for {} without a preceding /api/ include'.format(entity))
+                               f'/validity/ include found for {entity} without a preceding /api/ include')
                     return
 
                 if self.pname_mentions[entity]:
@@ -432,8 +459,8 @@ class MacroCheckerFile(object):
                                if member not in self.pname_mentions[entity]]
                     if missing:
                         self.error(MessageId.UNDOCUMENTED_MEMBER,
-                                   ['Validity include found for {}, but not all members/params apparently documented'.format(entity),
-                                    'Members/params not mentioned with pname: {}'.format(', '.join(missing))])
+                                   [f'Validity include found for {entity}, but not all members/params apparently documented',
+                                    f"Members/params not mentioned with pname: {', '.join(missing)}"])
 
             # If we found an include line, we're done with this line.
             return
@@ -464,8 +491,7 @@ class MacroCheckerFile(object):
                 # Got :: but not followed by pname
 
                 search = match.group()
-                replacement = match.group(
-                    'first_part') + '::pname:' + match.group('second_part')
+                replacement = f"{match.group('first_part')}::pname:{match.group('second_part')}"
                 self.error(MessageId.MEMBER_PNAME_MISSING,
                            'Found a function parameter or struct member reference with :: but missing pname:',
                            group='double_colons',
@@ -475,7 +501,7 @@ class MacroCheckerFile(object):
                 # check pname here because it won't come up in normal iteration below
                 # because of the missing macro
                 self.entity = match.group('entity_name')
-                self.checkPname(match.group('scope'))
+                self.checkPname(match.group('scope'), fully_qualified=True)
 
         ###
         # Look for things that seem like a missing macro.
@@ -501,7 +527,7 @@ class MacroCheckerFile(object):
                     macro = data.macro
                     category = data.category
                     self.warning(MessageId.MISSING_MACRO,
-                                 ['Seems like a "{}" macro was omitted for this reference to a known entity in category "{}".'.format(macro, category),
+                                 [f'Seems like a "{macro}" macro was omitted for this reference to a known entity in category "{category}".',
                                   'Wrap in ` ` to silence this if you do not want a verified macro here.'],
                                  group='entity_name',
                                  replacement=self.makeMacroMarkup(data.macro))
@@ -691,7 +717,7 @@ class MacroCheckerFile(object):
             scope = PRECEDING_MEMBER_REFERENCE.search(preceding)
             if scope:
                 # Yes there is, check it out.
-                self.checkPname(scope.group('entity_name'))
+                self.checkPname(scope.group('entity_name'), fully_qualified=True)
             elif self.current_ref_page is not None:
                 # No, but there is a current ref page: very reliable
                 self.checkPnameImpliedContext(self.current_ref_page)
@@ -703,6 +729,10 @@ class MacroCheckerFile(object):
                 # can't check this.
                 pass
 
+    def shouldSkipUnrecognizedEntity(self, macro,  entity_name):
+        """Return True if we should not warn about not recognizing a macro invocation for entity_name."""
+        return False
+
     def checkRecognizedEntity(self):
         """Check the current macro:entity match to see if it is recognized.
 
@@ -728,11 +758,13 @@ class MacroCheckerFile(object):
             _pluralize('category', len(possibleCats)),
             ', '.join(possibleCats))]
 
+        if self.shouldSkipUnrecognizedEntity(macro, entity):
+            return False
+
         data = self.checker.findEntity(entity)
         if data:
             # We found the goof: incorrect macro
-            msg.append('Apparently matching entity in category {} found.'.format(
-                data.category))
+            msg.append(f'Apparently matching entity in category {data.category} found.')
             self.handleWrongMacro(msg, data)
             return True
 
@@ -770,7 +802,7 @@ class MacroCheckerFile(object):
             else:
                 # Doesn't match our pattern,
                 # so probably should be name instead of link.
-                newMacro = macro[0] + 'name'
+                newMacro = f"{macro[0]}name"
                 if self.checker.entity_db.isValidMacro(newMacro):
                     self.error(MessageId.BAD_ENTITY, msg +
                                ['Entity name does not fit the pattern for this API, which would mean it should be a "name" macro instead of a "link" macro'],
@@ -778,7 +810,7 @@ class MacroCheckerFile(object):
                 else:
                     self.error(MessageId.BAD_ENTITY, msg +
                                ['Entity name does not fit the pattern for this API, which would mean it should be a "name" macro instead of a "link" macro',
-                                'However, {} is not a known macro so cannot auto-fix.'.format(newMacro)], see_also=see_also)
+                                f'However, {newMacro} is not a known macro so cannot auto-fix.'], see_also=see_also)
 
         elif macro == 'ename':
             # TODO This might be an ambiguity in the style guide - ename might be a known enumerant value,
@@ -787,7 +819,7 @@ class MacroCheckerFile(object):
             if self.checker.likelyRecognizedEntity(entity):
                 if not self.checkText():
                     self.warning(MessageId.BAD_ENUMERANT, msg +
-                                 ['Unrecognized ename:{} that we would expect to recognize since it fits the pattern for this API.'.format(entity)],
+                                 [f'Unrecognized ename:{entity} that we would expect to recognize since it fits the pattern for this API.'],
                                  see_also=see_also)
         else:
             # This is fine:
@@ -809,26 +841,28 @@ class MacroCheckerFile(object):
         shouldBeText = shouldEntityBeText(entity, self.subscript)
         assert macro
         if shouldBeText and not macro.endswith('text') and not macro == 'code':
-            newMacro = macro[0] + 'text'
+            newMacro = f"{macro[0]}text"
             if self.checker.entity_db.getCategoriesForMacro(newMacro):
                 self.error(MessageId.MISSING_TEXT,
-                           ['Asterisk/leading or trailing underscore/bracket found - macro should end with "text:", probably {}:'.format(newMacro),
+                           [f'Asterisk/leading or trailing underscore/bracket found - macro should end with "text:", probably {newMacro}:',
                             AUTO_FIX_STRING],
                            group='macro', replacement=newMacro, fix=self.makeFix(newMacro=newMacro))
             else:
                 self.error(MessageId.MISSING_TEXT,
                            ['Asterisk/leading or trailing underscore/bracket found, so macro should end with "text:".',
-                            'However {}: is not a known macro so cannot auto-fix.'.format(newMacro)],
+                            f'However {newMacro}: is not a known macro so cannot auto-fix.'],
                            group='macro')
             return True
         elif macro.endswith('text') and not shouldBeText:
             msg = [
-                "No asterisk/leading or trailing underscore/bracket in the entity, so this might be a mistaken use of the 'text' macro {}:".format(macro)]
+                f"No asterisk/leading or trailing underscore/bracket in the entity, so this might be a mistaken use of the 'text' macro {macro}:"]
             data = self.checker.findEntity(entity)
             if data:
+                if self.shouldSkipUnrecognizedEntity(macro, entity):
+                    return False
+
                 # We found the goof: incorrect macro
-                msg.append('Apparently matching entity in category {} found.'.format(
-                    data.category))
+                msg.append(f'Apparently matching entity in category {data.category} found.')
                 msg.append(AUTO_FIX_STRING)
                 replacement = self.makeFix(data=data)
                 if data.category == EXTENSION_CATEGORY:
@@ -847,13 +881,13 @@ class MacroCheckerFile(object):
                     # Only suggest a macro if we aren't in elink/ename/etext,
                     # since ename and elink are not related in an equivalent way
                     # to the relationship between flink and fname.
-                    newMacro = macro[0] + 'name'
+                    newMacro = f"{macro[0]}name"
                     if self.checker.entity_db.getCategoriesForMacro(newMacro):
                         msg.append(
-                            'Consider if {}: might be the correct macro to use here.'.format(newMacro))
+                            f'Consider if {newMacro}: might be the correct macro to use here.')
                     else:
                         msg.append(
-                            'Cannot suggest a new macro because {}: is not a known macro.'.format(newMacro))
+                            f'Cannot suggest a new macro because {newMacro}: is not a known macro.')
                 self.warning(MessageId.MISUSED_TEXT, msg)
             return True
         return False
@@ -873,7 +907,7 @@ class MacroCheckerFile(object):
             # in case we're in the documentation block.
             self.pname_mentions[pname_context.entity].add(self.entity)
 
-    def checkPname(self, pname_context):
+    def checkPname(self, pname_context, fully_qualified=False):
         """Check the current match (as a pname: usage) with the given entity as its 'pname context', if possible.
 
         e.g. slink:foo::pname:bar, pname_context would be 'foo', while self.entity would be 'bar', etc.
@@ -897,12 +931,19 @@ class MacroCheckerFile(object):
             return
         if not members:
             self.warning(MessageId.UNRECOGNIZED_CONTEXT,
-                         'pname context entity was un-recognized {}'.format(pname_context))
+                         f'pname context entity was un-recognized {pname_context}')
             return
 
+        assert entity
+
+        # Strip trailing array dimension
+        if ARRAY_DIMENSION.search(entity):
+            entity = ARRAY_DIMENSION.sub("", entity)
+
         if entity not in members:
-            self.warning(MessageId.UNKNOWN_MEMBER, ["Could not find member/param named '{}' in {}".format(entity, pname_context),
-                                                    'Known {} mamber/param names are: {}'.format(
+            message_id = MessageId.UNKNOWN_MEMBER_FULLY_QUALIFIED if fully_qualified else MessageId.UNKNOWN_MEMBER
+            self.warning(message_id, ["Could not find member/param named '{}' in {}".format(entity, pname_context),
+                                                    'Known {} member/param names are: {}'.format(
                 pname_context, ', '.join(members))], group='entity_name')
 
     def checkIncludeRefPageRelation(self, entity, generated_type):
@@ -981,7 +1022,7 @@ class MacroCheckerFile(object):
             VALID_REF_PAGE_ATTRIBS)
         if unknown_attribs:
             self.error(MessageId.REFPAGE_UNKNOWN_ATTRIB,
-                       "Found unknown attrib(s) in reference page markup: " + ','.join(unknown_attribs))
+                       f"Found unknown attrib(s) in reference page markup: {','.join(unknown_attribs)}")
 
         # Required field: refpage='xrValidEntityHere'
         if Attrib.REFPAGE.value in attribs:
@@ -993,8 +1034,7 @@ class MacroCheckerFile(object):
                 group='value', match=attrib.match)
             if self.checker.seenRefPage(text):
                 self.error(MessageId.REFPAGE_DUPLICATE,
-                           ["Found reference page markup when we already saw refpage='{}' elsewhere.".format(
-                               text),
+                           [f"Found reference page markup when we already saw refpage='{text}' elsewhere.",
                             "This (or the other mention) may be a copy-paste error."],
                            context=context)
             self.checker.addRefPage(text)
@@ -1047,8 +1087,7 @@ class MacroCheckerFile(object):
                 context = self.storeMessageContext(
                     group='value', match=attrib.match)
                 self.error(MessageId.REFPAGE_TYPE,
-                           "Found reference page markup, but type='{}' is not the expected value '{}'".format(
-                               text, directory),
+                           f"Found reference page markup, but type='{text}' is not the expected value '{directory}'",
                            context=context)
         else:
             self.error(MessageId.REFPAGE_TAG,
@@ -1115,8 +1154,7 @@ class MacroCheckerFile(object):
             old_text = text
             text = remakeRefs(unique_refs.keys())
             self.warning(MessageId.REFPAGE_XREF_DUPE,
-                         ["Reference page for {} contains at least one duplicate in its cross-references.".format(
-                             self.entity),
+                         [f"Reference page for {self.entity} contains at least one duplicate in its cross-references.",
                              "Look carefully to see if this is a copy and paste error and should be changed to a different but related entity:",
                              "auto-fix simply removes the duplicate."],
                          context=context,
@@ -1131,7 +1169,7 @@ class MacroCheckerFile(object):
             # DON'T AUTOFIX HERE because these are likely copy-paste between related entities:
             # e.g. a Create function and the associated CreateInfo struct.
             self.warning(MessageId.REFPAGE_SELF_XREF,
-                         ["Reference page for {} included itself in its cross-references.".format(self.entity),
+                         [f"Reference page for {self.entity} included itself in its cross-references.",
                           "This is typically a copy and paste error, and the dupe should likely be changed to a different but related entity.",
                           "Not auto-fixing for this reason."],
                          context=context,
@@ -1144,7 +1182,7 @@ class MacroCheckerFile(object):
             text = remakeRefs(unique_refs.keys())
             if old_text != text:
                 self.warning(MessageId.REFPAGE_WHITESPACE,
-                             ["Cross-references for reference page for {} had non-minimal whitespace,".format(self.entity),
+                             [f"Cross-references for reference page for {self.entity} had non-minimal whitespace,",
                               "and no other enabled message has re-constructed this value already."],
                              context=context,
                              replacement=text,
@@ -1205,8 +1243,7 @@ class MacroCheckerFile(object):
                 data = dataArray[0]
                 new_entity = data.entity
                 self.error(MessageId.REFPAGE_XREFS, msg + [
-                    'Apparently matching entity in category {} found by searching case-insensitively.'.format(
-                        data.category),
+                    f'Apparently matching entity in category {data.category} found by searching case-insensitively.',
                     AUTO_FIX_STRING],
                     replacement=new_entity,
                     fix=(referenced_entity, new_entity),
@@ -1385,8 +1422,7 @@ class MacroCheckerFile(object):
         for message_type in [MessageType.ERROR, MessageType.WARNING]:
             count = self.numMessagesOfType(message_type)
             if count > 0:
-                print('{num} {mtype}{s} generated.'.format(
-                    num=count, mtype=message_type, s=_s_suffix(count)))
+                print(f'{count} {message_type}{_s_suffix(count)} generated.')
 
     def dumpInternals(self):
         """Dump internal variables to screen, for debugging."""
@@ -1488,9 +1524,8 @@ class MacroCheckerFile(object):
     def getBriefLocation(self, context):
         """Format a context briefly - omitting the filename if it has newlines in it."""
         if '\n' in context.filename:
-            return 'input string line {}'.format(context.lineNum)
-        return '{}:{}'.format(
-            context.filename, context.lineNum)
+            return f'input string line {context.lineNum}'
+        return f'{context.filename}:{context.lineNum}'
 
     ###
     # Handlers for a variety of diagnostic-meriting conditions
@@ -1500,7 +1535,7 @@ class MacroCheckerFile(object):
 
     def handleIncludeMissingRefPage(self, entity, generated_type):
         """Report a message about an include outside of a ref-page block."""
-        msg = ["Found {} include for {} outside of a reference page block.".format(generated_type, entity),
+        msg = [f"Found {generated_type} include for {entity} outside of a reference page block.",
                "This is probably a missing reference page block."]
         refpage = self.computeExpectedRefPageFromInclude(entity)
         data = self.checker.findEntity(refpage)
@@ -1509,7 +1544,7 @@ class MacroCheckerFile(object):
             msg.append(self.makeRefPageTag(refpage, data=data))
         else:
             msg.append(
-                "But, expected ref page entity name {} isn't recognized...".format(refpage))
+                f"But, expected ref page entity name {refpage} isn't recognized...")
         self.warning(MessageId.REFPAGE_MISSING, msg)
 
     def handleIncludeMismatchRefPage(self, entity, generated_type):
@@ -1583,13 +1618,13 @@ class MacroCheckerFile(object):
 
     def makeSearch(self):
         """Construct the string self.macro:self.entity, for use in the old text part of a fix pair."""
-        return '{}:{}'.format(self.macro, self.entity)
+        return f'{self.macro}:{self.entity}'
 
     def makeMacroMarkup(self, newMacro=None, newEntity=None, data=None):
         """Construct appropriate markup for referring to an entity.
 
-        Typically constructs macro:entity, but can construct apiext:EXTENSION_NAME if the supplied
-        entity is identified as an extension.
+        Typically constructs macro:entity, but can construct `<<EXTENSION_NAME>>`
+        (or equivalent) if the supplied entity is identified as an extension.
 
         Arguments:
         newMacro -- The macro to use. Defaults to data.macro (if available), otherwise self.macro.
@@ -1610,19 +1645,19 @@ class MacroCheckerFile(object):
             data = self.checker.findEntity(newEntity)
         if data and data.category == EXTENSION_CATEGORY:
             return self.makeExtensionLink(newEntity)
-        return '{}:{}'.format(newMacro, newEntity)
+        return f'{newMacro}:{newEntity}'
 
     def makeExtensionLink(self, newEntity=None):
         """Create a correctly-formatted link to an extension.
 
-        Result takes the form apiext:EXTENSION_NAME.
+        Result takes the form `<<EXTENSION_NAME>>` or whatever the Conventions define.
 
         Argument:
         newEntity -- The extension name to link to. Defaults to self.entity.
         """
         if not newEntity:
             newEntity = self.entity
-        return 'apiext:{}'.format(newEntity)
+        return self.checker.conventions().formatExtension(newEntity)
 
     def computeExpectedRefPageFromInclude(self, entity):
         """Compute the expected ref page entity based on an include entity name."""
