@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2024, The Khronos Group Inc.
+// Copyright (c) 2017-2025 The Khronos Group Inc.
 // Copyright (c) 2017-2019 Valve Corporation
 // Copyright (c) 2017-2019 LunarG, Inc.
 //
@@ -32,9 +32,12 @@
 
 #include <algorithm>
 #include <cctype>
+#include <chrono>
 #include <cstring>
 #include <fstream>
+#include <iomanip>
 #include <iostream>
+#include <inttypes.h>
 #include <memory>
 #include <mutex>
 #include <sstream>
@@ -44,6 +47,10 @@
 #include <utility>
 #include <vector>
 
+#ifdef __ANDROID__
+#include "android/log.h"
+#endif
+
 #if defined(__GNUC__) && __GNUC__ >= 4
 #define LAYER_EXPORT __attribute__((visibility("default")))
 #elif defined(__SUNPRO_C) && (__SUNPRO_C >= 0x590)
@@ -77,114 +84,117 @@ bool CoreValidationWriteHtmlHeader() {
         std::unique_lock<std::mutex> mlock(g_record_mutex);
         std::ofstream html_file;
         html_file.open(g_record_info.file_name, std::ios::out);
-        html_file
-            << "<!doctype html>\n"
-               "<html>\n"
-               "    <head>\n"
-               "        <title>OpenXR Core Validation</title>\n"
-               "        <style type='text/css'>\n"
-               "        html {\n"
-               "            background-color: #0b1e48;\n"
-               "            background-image: url('https://vulkan.lunarg.com/img/bg-starfield.jpg');\n"
-               "            background-position: center;\n"
-               "            -webkit-background-size: cover;\n"
-               "            -moz-background-size: cover;\n"
-               "            -o-background-size: cover;\n"
-               "            background-size: cover;\n"
-               "            background-attachment: fixed;\n"
-               "            background-repeat: no-repeat;\n"
-               "            height: 100%;\n"
-               "        }\n"
-               "        #header {\n"
-               "            z-index: -1;\n"
-               "        }\n"
-               "        #header>img {\n"
-               "            position: absolute;\n"
-               "            width: 160px;\n"
-               "            margin-left: -280px;\n"
-               "            top: -10px;\n"
-               "            left: 50%;\n"
-               "        }\n"
-               "        #header>h1 {\n"
-               "            font-family: Arial, 'Helvetica Neue', Helvetica, sans-serif;\n"
-               "            font-size: 48px;\n"
-               "            font-weight: 200;\n"
-               "            text-shadow: 4px 4px 5px #000;\n"
-               "            color: #eee;\n"
-               "            position: absolute;\n"
-               "            width: 600px;\n"
-               "            margin-left: -80px;\n"
-               "            top: 8px;\n"
-               "            left: 50%;\n"
-               "        }\n"
-               "        body {\n"
-               "            font-family: Consolas, monaco, monospace;\n"
-               "            font-size: 14px;\n"
-               "            line-height: 20px;\n"
-               "            color: #eee;\n"
-               "            height: 100%;\n"
-               "            margin: 0;\n"
-               "            overflow: hidden;\n"
-               "        }\n"
-               "        #wrapper {\n"
-               "            background-color: rgba(0, 0, 0, 0.7);\n"
-               "            border: 1px solid #446;\n"
-               "            box-shadow: 0px 0px 10px #000;\n"
-               "            padding: 8px 12px;\n"
-               "            display: inline-block;\n"
-               "            position: absolute;\n"
-               "            top: 80px;\n"
-               "            bottom: 25px;\n"
-               "            left: 50px;\n"
-               "            right: 50px;\n"
-               "            overflow: auto;\n"
-               "        }\n"
-               "        details>*:not(summary) {\n"
-               "            margin-left: 22px;\n"
-               "        }\n"
-               "        summary:only-child {\n"
-               "            display: block;\n"
-               "            padding-left: 15px;\n"
-               "        }\n"
-               "        details>summary:only-child::-webkit-details-marker {\n"
-               "            display: none;\n"
-               "            padding-left: 15px;\n"
-               "        }\n"
-               "        .headervar, .generalheadertype, .warningheadertype, .errorheadertype, .debugheadertype, .headerval {\n"
-               "            display: inline;\n"
-               "            margin: 0 9px;\n"
-               "        }\n"
-               "        .var, .type, .val {\n"
-               "            display: inline;\n"
-               "            margin: 0 6px;\n"
-               "        }\n"
-               "        .warningheadertype, .type {\n"
-               "            color: #dce22f;\n"
-               "        }\n"
-               "        .errorheadertype, .type {\n"
-               "            color: #ff1616;\n"
-               "        }\n"
-               "        .debugheadertype, .type {\n"
-               "            color: #888;\n"
-               "        }\n"
-               "        .generalheadertype, .type {\n"
-               "            color: #acf;\n"
-               "        }\n"
-               "        .headerval, .val {\n"
-               "            color: #afa;\n"
-               "            text-align: right;\n"
-               "        }\n"
-               "        .thd {\n"
-               "            color: #888;\n"
-               "        }\n"
-               "        </style>\n"
-               "    </head>\n"
-               "    <body>\n"
-               "        <div id='header'>\n"
-               "            <img src='https://lunarg.com/wp-content/uploads/2016/02/LunarG-wReg-150.png' />\n"
-               "            <h1>OpenXR Core Validation</h1>\n"
-               "        </div>\n"
-               "        <div id='wrapper'>\n";
+        html_file << "<!doctype html>\n"
+                     "<html>\n"
+                     "    <head>\n"
+                     "        <title>OpenXR Core Validation</title>\n"
+                     "        <style type='text/css'>\n"
+                     "        html {\n"
+                     "            background-color: #0b1e48;\n"
+                     "            background-image: url('https://vulkan.lunarg.com/img/bg-starfield.jpg');\n"
+                     "            background-position: center;\n"
+                     "            -webkit-background-size: cover;\n"
+                     "            -moz-background-size: cover;\n"
+                     "            -o-background-size: cover;\n"
+                     "            background-size: cover;\n"
+                     "            background-attachment: fixed;\n"
+                     "            background-repeat: no-repeat;\n"
+                     "            height: 100%;\n"
+                     "        }\n"
+                     "        #header {\n"
+                     "            z-index: -1;\n"
+                     "        }\n"
+                     "        #header>img {\n"
+                     "            position: absolute;\n"
+                     "            width: 160px;\n"
+                     "            margin-left: -280px;\n"
+                     "            top: -10px;\n"
+                     "            left: 50%;\n"
+                     "        }\n"
+                     "        #header>h1 {\n"
+                     "            font-family: Arial, 'Helvetica Neue', Helvetica, sans-serif;\n"
+                     "            font-size: 48px;\n"
+                     "            font-weight: 200;\n"
+                     "            text-shadow: 4px 4px 5px #000;\n"
+                     "            color: #eee;\n"
+                     "            position: absolute;\n"
+                     "            width: 600px;\n"
+                     "            margin-left: -80px;\n"
+                     "            top: 8px;\n"
+                     "            left: 50%;\n"
+                     "        }\n"
+                     "        body {\n"
+                     "            font-family: Consolas, monaco, monospace;\n"
+                     "            font-size: 14px;\n"
+                     "            line-height: 20px;\n"
+                     "            color: #eee;\n"
+                     "            height: 100%;\n"
+                     "            margin: 0;\n"
+                     "            overflow: hidden;\n"
+                     "        }\n"
+                     "        #wrapper {\n"
+                     "            background-color: rgba(0, 0, 0, 0.7);\n"
+                     "            border: 1px solid #446;\n"
+                     "            box-shadow: 0px 0px 10px #000;\n"
+                     "            padding: 8px 12px;\n"
+                     "            display: inline-block;\n"
+                     "            position: absolute;\n"
+                     "            top: 80px;\n"
+                     "            bottom: 25px;\n"
+                     "            left: 50px;\n"
+                     "            right: 50px;\n"
+                     "            overflow: auto;\n"
+                     "        }\n"
+                     "        details>*:not(summary) {\n"
+                     "            margin-left: 22px;\n"
+                     "        }\n"
+                     "        summary:only-child {\n"
+                     "            display: block;\n"
+                     "            padding-left: 15px;\n"
+                     "        }\n"
+                     "        details>summary:only-child::-webkit-details-marker {\n"
+                     "            display: none;\n"
+                     "            padding-left: 15px;\n"
+                     "        }\n"
+                     "        .headervar, .generalheadertype, .warningheadertype, .errorheadertype, .debugheadertype, .headerval, "
+                     ".timestampval {\n"
+                     "            display: inline;\n"
+                     "            margin: 0 9px;\n"
+                     "        }\n"
+                     "        .var, .type, .val {\n"
+                     "            display: inline;\n"
+                     "            margin: 0 6px;\n"
+                     "        }\n"
+                     "        .warningheadertype, .type {\n"
+                     "            color: #dce22f;\n"
+                     "        }\n"
+                     "        .errorheadertype, .type {\n"
+                     "            color: #ff1616;\n"
+                     "        }\n"
+                     "        .debugheadertype, .type {\n"
+                     "            color: #888;\n"
+                     "        }\n"
+                     "        .generalheadertype, .type {\n"
+                     "            color: #acf;\n"
+                     "        }\n"
+                     "        .headerval, .val {\n"
+                     "            color: #afa;\n"
+                     "            text-align: right;\n"
+                     "        }\n"
+                     "        .timestampval {\n"
+                     "            color: #888;\n"
+                     "        }\n"
+                     "        .thd {\n"
+                     "            color: #888;\n"
+                     "        }\n"
+                     "        </style>\n"
+                     "    </head>\n"
+                     "    <body>\n"
+                     "        <div id='header'>\n"
+                     "            <img src='https://lunarg.com/wp-content/uploads/2016/02/LunarG-wReg-150.png' />\n"
+                     "            <h1>OpenXR Core Validation</h1>\n"
+                     "        </div>\n"
+                     "        <div id='wrapper'>\n";
         return true;
     } catch (...) {
         return false;
@@ -213,13 +223,26 @@ bool CoreValidationWriteHtmlFooter() {
 
 // For routing platform_utils.hpp messages.
 void LogPlatformUtilsError(const std::string &message) {
-    (void)message;
+    (void)message;  // maybe unused
 #if !defined(NDEBUG)
     std::cerr << message << std::endl;
+#endif
+
 #if defined(XR_OS_WINDOWS)
     OutputDebugStringA((message + "\n").c_str());
+#elif defined(XR_OS_ANDROID)
+    __android_log_write(ANDROID_LOG_ERROR, "OpenXR-CoreValidation", message.c_str());
 #endif
-#endif
+}
+
+// Get the current time as a string
+std::string GenerateTimestamp() {
+    auto now = std::chrono::system_clock::now();
+    std::time_t now_c = std::chrono::system_clock::to_time_t(now);
+    std::tm local_tm = *std::localtime(&now_c);
+    std::ostringstream oss;
+    oss << std::put_time(&local_tm, "%Y-%m-%d %H:%M:%S");
+    return oss.str();
 }
 
 // Function to record all the core validation information
@@ -229,6 +252,9 @@ void CoreValidLogMessage(GenValidUsageXrInstanceInfo *instance_info, const std::
     if (g_record_info.initialized) {
         std::unique_lock<std::mutex> mlock(g_record_mutex);
 
+        // Generate a timestamp
+        std::string timestamp = GenerateTimestamp();
+
         // Debug Utils items (in case we need them)
         XrDebugUtilsMessageSeverityFlagsEXT debug_utils_severity = 0;
 
@@ -294,33 +320,39 @@ void CoreValidLogMessage(GenValidUsageXrInstanceInfo *instance_info, const std::
 
         switch (g_record_info.type) {
             case RECORD_TEXT_COUT: {
-                std::cout << "[" << severity_string << " | " << message_id << " | " << command_name << "]: " << message
-                          << std::endl;
+#if defined(ANDROID)
+#define ALOGI(...)       \
+    printf(__VA_ARGS__); \
+    __android_log_print(ANDROID_LOG_INFO, "core_validation", __VA_ARGS__)
+#else
+#define ALOGI(...) printf(__VA_ARGS__)
+#endif
+                ALOGI("[%s][%s|%s|%s]: %s \n", timestamp.c_str(), severity_string.c_str(), message_id.c_str(), command_name.c_str(),
+                      message.c_str());
                 if (!objects_info.empty()) {
-                    std::cout << "  Objects:" << std::endl;
+                    ALOGI("  Objects:\n");
                     uint32_t count = 0;
                     for (const auto &object_info : objects_info) {
                         std::string object_type = GenValidUsageXrObjectTypeToString(object_info.type);
-                        std::ostringstream oss_object_handle;
-                        std::cout << "   [" << std::to_string(count++) << "] - " << object_type << " ("
-                                  << Uint64ToHexString(object_info.handle) << ")";
-                        std::cout << std::endl;
+                        std::string hexString = Uint64ToHexString(object_info.handle);
+                        ALOGI("   [%" PRIu32 "] - %s (%s)\n", count++, object_type.c_str(), hexString.c_str());
                     }
                 }
                 if (!names_and_labels.labels.empty()) {
-                    std::cout << "  Session Labels:" << std::endl;
+                    ALOGI("  Session Labels:\n");
                     uint32_t count = 0;
                     for (const auto &session_label : names_and_labels.labels) {
-                        std::cout << "   [" << std::to_string(count++) << "] - " << session_label.labelName << std::endl;
+                        ALOGI("   [%" PRIu32 "] - %s\n", count++, session_label.labelName);
+#undef ALOGI
                     }
                 }
-                std::cout << std::flush;
                 break;
             }
             case RECORD_TEXT_FILE: {
                 std::ofstream text_file;
                 text_file.open(g_record_info.file_name, std::ios::out | std::ios::app);
-                text_file << "[" << severity_string << " | " << message_id << " | " << command_name << "]: " << message
+                text_file << "[" << timestamp << "]"
+                          << "[" << severity_string << " | " << message_id << " | " << command_name << "] : " << message
                           << std::endl;
                 if (!objects_info.empty()) {
                     text_file << "  Objects:" << std::endl;
@@ -369,6 +401,7 @@ void CoreValidLogMessage(GenValidUsageXrInstanceInfo *instance_info, const std::
                         break;
                 }
                 text_file << "   <summary>\n"
+                          << "      <div class='timestampval'>[" << timestamp << "]</div>\n"
                           << "      <div class='" << header_type << "'>" << severity_string << "</div>\n"
                           << "      <div class='headerval'>" << command_name << "</div>\n"
                           << "      <div class='headervar'>" << message_id << "</div>\n"
@@ -497,8 +530,16 @@ XRAPI_ATTR XrResult XRAPI_CALL CoreValidationXrCreateApiLayerInstance(const XrIn
             g_record_info.type = RECORD_TEXT_COUT;
         }
 
+#if !defined(ANDROID)
         std::string export_type = PlatformUtilsGetEnv("XR_CORE_VALIDATION_EXPORT_TYPE");
         std::string file_name = PlatformUtilsGetEnv("XR_CORE_VALIDATION_FILE_NAME");
+#else
+        // We match the pattern used by the Vulkan api_dump layer here
+        // (we replace the `XR_` prefix with `debug.` and make it lowercase.)
+        // adb shell "setprop debug.api_dump_file_name '/sdcard/xr_apidump.txt'"
+        std::string export_type = PlatformUtilsGetAndroidSystemProperty("debug.core_validation_export_type");
+        std::string file_name = PlatformUtilsGetAndroidSystemProperty("debug.core_validation_file_name");
+#endif
         if (!file_name.empty()) {
             g_record_info.file_name = file_name;
             g_record_info.type = RECORD_TEXT_FILE;
@@ -522,6 +563,9 @@ XRAPI_ATTR XrResult XRAPI_CALL CoreValidationXrCreateApiLayerInstance(const XrIn
         std::cerr << "Core Validation output type: " << (export_type.empty() ? "text" : export_type)
                   << ", first time = " << (first_time ? "true" : "false") << std::endl;
 
+        CoreValidLogMessage(nullptr, "VUID-CoreValidation-Initialize", VALID_USAGE_DEBUG_SEVERITY_DEBUG, "xrCreateApiLayerInstance",
+                            std::vector<GenValidUsageXrObjectInfo>(), "Core Validation Layer is initialized");
+
         // Call the generated pre valid usage check.
         validation_result = GenValidUsageInputsXrCreateInstance(info, instance);
 
@@ -600,6 +644,10 @@ XRAPI_ATTR XrResult XRAPI_CALL CoreValidationXrDestroyInstance(XrInstance instan
             gen_instance_info->debug_messengers.clear();
         }
     }
+
+    CoreValidLogMessage(nullptr, "VUID-CoreValidation-Destroy", VALID_USAGE_DEBUG_SEVERITY_DEBUG, "xrDestroyInstance",
+                        std::vector<GenValidUsageXrObjectInfo>(), "Core Validation Layer will be destroyed");
+
     XrResult result = GenValidUsageNextXrDestroyInstance(instance);
     if (!g_instance_info.empty() && g_record_info.type == RECORD_HTML_FILE) {
         CoreValidationWriteHtmlFooter();
